@@ -1,22 +1,19 @@
 
 import os
 import sys
-from collections import OrderedDict
 
-python_version      = sys.version_info
 is_pypy_interpreter = ('__pypy__' in sys.builtin_module_names)
 is_windows_os       = (os.name == 'nt')
-is_tty_console      = (hasattr(sys.stdout, 'isatty') and sys.stdout.isatty())
 is_utf_console      = ('utf' in sys.stdout.encoding.lower())
 
 ''' ordereddict:
-    starting at python 3.6 the built-in dict is both 
-    insertion-ordered AND compact, using about half the 
-    memory of collections.OrderedDict
+    starting at python 3.6, the built-in dict is both insertion-ordered and compact, 
+    using about half the memory of collections.OrderedDict
 '''
-if python_version >= (3, 6):
+if sys.version_info >= (3, 6):
     ordereddict = dict
 else:
+    from collections import OrderedDict
     ordereddict = OrderedDict
 
 try:
@@ -48,14 +45,14 @@ try:
 except ImportError:
     line_profiler_installed = False
 
-# is_pypy_interpreter = True
-# is_windows_os       = True
+# determines if excel_com module should be loaded in vengeance.__init__
+loads_excel_module = (not is_pypy_interpreter) and is_windows_os
+
+# is_pypy_interpreter     = True
+# is_windows_os           = True
 # dateutil_installed      = False
 # ultrajson_installed     = False
 # line_profiler_installed = False
-
-# determines if excel_com module should be loaded in __init__
-loads_excel_module = (not is_pypy_interpreter) and is_windows_os
-pass
+# loads_excel_module      = False
 
 
