@@ -1,26 +1,26 @@
 
 import ctypes
+import gc
 import os
 import pythoncom
 
+# noinspection PyUnresolvedReferences
+from pythoncom import com_error as pythoncom_error
 from _ctypes import COMError as ctypes_error
-from ctypes import POINTER
-from ctypes import PyDLL
+
 from ctypes import byref
 from ctypes import c_void_p
 from ctypes import py_object
+from ctypes import POINTER
+from ctypes import PyDLL
 from ctypes.wintypes import BOOL
 
-from comtypes import GUID
 from comtypes import IUnknown
-
 from comtypes.client          import CreateObject as comtypes_createobject
 from comtypes.automation      import IDispatch    as comtypes_idispatch
 from comtypes.client.dynamic  import Dispatch     as comtypes_dispatch
-from win32com.client.gencache import EnsureDispatch                          # early-bound references
 
-# noinspection PyUnresolvedReferences
-from pythoncom import com_error as pythoncom_error
+from win32com.client.gencache import EnsureDispatch                          # early-bound references
 
 from ..util.filesystem import standardize_path
 from ..util.filesystem import validate_path_exists
@@ -28,7 +28,8 @@ from ..util.filesystem import validate_path_exists
 from ..util.text import styled
 from ..util.text import vengeance_message
 from ..util.text import vengeance_warning
-from .excel_constants import *
+
+from .excel_constants import xlMaximized
 
 # Windows api functions
 FindWindowExA              = ctypes.windll.user32.FindWindowExA
@@ -40,20 +41,17 @@ xl_main_ascii = 'XLMAIN'.encode('ascii')
 xl_desk_ascii = 'XLDESK'.encode('ascii')
 excel7_ascii  = 'EXCEL7'.encode('ascii')
 
-xl_clsid      = '{00020400-0000-0000-C000-000000000046}'
 corrupt_hwnds = set()
 native_om     = -16
 
 
-def open_workbook(path,
-                  excel_app='new',
+def open_workbook(path, excel_app='new',
                   *,
                   read_only=False,
                   update_links=True,
                   windowstate=xlMaximized):
 
     wb = get_opened_workbook(path)
-
     if wb is None:
         wb = __open_workbook_dispatch(path,
                                       excel_app,
@@ -109,28 +107,24 @@ def close_workbook(wb, save):
         1) if Workbook is the only one in the Excel application, closes the application as well
         2) correctly releases the com pointer for the workbook
     """
-    import gc
-
     if save and wb.ReadOnly:
         raise AssertionError("'{}' is open as read-only, cannot save and close".format(wb.Name))
 
     excel_app = wb.Application
+
     display_alerts = excel_app.DisplayAlerts
-
-    if save:
-        wb.Save()
-
     excel_app.DisplayAlerts = False
-    wb.Close()
+    wb.Close(save)
+    excel_app.DisplayAlerts = display_alerts
+
     wb = None
     del wb
 
     if excel_app.Workbooks.Count == 0:
         excel_app.Quit()
+
         excel_app = None
         del excel_app
-    else:
-        excel_app.DisplayAlerts = display_alerts
 
     gc.collect()
 
@@ -182,9 +176,9 @@ def __validate_excel_application(excel_app, windowstate=None):
     return excel_app
 
 
-def new_excel_application(windowstate=None):
+def new_excel_application(windowstate=xlMaximized):
     excel_app = comtypes_createobject('Excel.Application', dynamic=True)
-    excel_app = __comtype_pointer_to_pythoncom_object(excel_app, comtypes_idispatch)
+    excel_app = __iunknown_pointer_to_python_object(excel_app, comtypes_idispatch)
     excel_app = EnsureDispatch(excel_app)
 
     reload_all_add_ins(excel_app)
@@ -203,7 +197,7 @@ def any_excel_application(windowstate=None):
     return excel_app
 
 
-def empty_excel_application(windowstate=None):
+def empty_excel_application(windowstate=xlMaximized):
     for excel_app in all_excel_instances():
 
         if __is_excel_application_empty(excel_app):
@@ -254,14 +248,26 @@ def reload_all_add_ins(excel_app):
     print()
 
 
+# noinspection PyUnusedLocal,PyBroadException
 def excel_application_to_foreground(excel_app, windowstate=None):
-    if windowstate not in (None, xlNormal, xlMaximized):
-        raise ValueError('windowstate must be in: (None, xlNormal, xlMaximized)')
-
     if windowstate is not None:
-        excel_app.WindowState = windowstate
+        try:
+            excel_app.WindowState = windowstate
+        except Exception:
+            pass
 
     excel_app.Visible = True
+
+    if excel_app.Visible is False and excel_app.Workbooks.Count == 0:
+        wb = excel_app.Workbooks.Add()
+        excel_app.Visible = True
+        wb.Close(False)
+
+        wb = None
+        del wb
+
+        gc.collect()
+
     SetForegroundWindow(excel_app.Hwnd)
 
 
@@ -269,6 +275,7 @@ def __next_window_handle(window_h=0):
     return FindWindowExA(0, window_h, xl_main_ascii, None)
 
 
+# noinspection PyTypeChecker,PyProtectedMember,PyUnresolvedReferences
 def __excel_application_from_window_handle(window_h):
     """
     comtypes library is used to search windows handles for Excel application,
@@ -278,57 +285,60 @@ def __excel_application_from_window_handle(window_h):
         Expected type [_CT], got Type[IDispatch] instead
         this is probably not correct way to set up this pointer
 
-        from pywintypes import IID
+        from pywintypes import IID?
     """
     global corrupt_hwnds
 
     if window_h in corrupt_hwnds:
         return None
 
-    desk_hwnd   = FindWindowExA(window_h,  None, xl_desk_ascii, None)
-    excel7_hwnd = FindWindowExA(desk_hwnd, None, excel7_ascii,  None)
+    xl_desk_hwnd = FindWindowExA(window_h,    None,  xl_desk_ascii, None)
+    excel7_hwnd  = FindWindowExA(xl_desk_hwnd, None, excel7_ascii,  None)
 
     if excel7_hwnd == 0:
         corrupt_hwnds.add(window_h)
         return None
 
-    # noinspection PyTypeChecker
     obj_ptr = POINTER(comtypes_idispatch)()
-    xl_guid = GUID.from_progid(xl_clsid)
     AccessibleObjectFromWindow(excel7_hwnd,
                                native_om,
-                               byref(xl_guid),
+                               byref(obj_ptr._iid_),
                                byref(obj_ptr))
 
-    # comtypes Dispatch() call rejected: user may be editing cell
     try:
         com_ptr   = comtypes_dispatch(obj_ptr).Application
-        excel_app = __comtype_pointer_to_pythoncom_object(com_ptr, comtypes_idispatch)
+        excel_app = __iunknown_pointer_to_python_object(com_ptr, comtypes_idispatch)
     except (ctypes_error, pythoncom_error, NameError):
+
+        # comtypes Dispatch() call rejected: user may be editing cell
         raise ChildProcessError('Remote Procedure Call to Excel Application rejected. '
                                 '\n'
                                 '\n\t(Excel will reject automation calls while waiting on user input, '
                                 '\n\tcheck if cursor is still active within a cell somewhere)') from None
 
-    # win32com Dispatch() call rejected: COM files may be corrupted
     try:
         return EnsureDispatch(excel_app)
     except AttributeError:
+
+        # win32com Dispatch() call rejected: COM files may be corrupted
         __move_win32com_gencache_folder()
+
         raise ChildProcessError('Error dispatching Excel Application from win32com module. '
                                 '\n'
-                                '\n\t(Deleting the contents of the win32com gen_py folder '
-                                'may resolve the error)') from None
+                                '\n\t(contents of the win32com gen_py folder may be corrupt)') from None
 
 
 # noinspection PyTypeChecker
-def __comtype_pointer_to_pythoncom_object(ptr, interface):
-    com_obj          = PyDLL(pythoncom.__file__).PyCom_PyObjectFromIUnknown
-    com_obj.restype  = py_object
-    com_obj.argtypes = (POINTER(IUnknown), c_void_p, BOOL)
+def __iunknown_pointer_to_python_object(com_ptr, interface):
+    function_pointer = PyDLL(pythoncom.__file__).PyCom_PyObjectFromIUnknown
+
+    function_pointer.restype  = py_object
+    function_pointer.argtypes = (POINTER(IUnknown),
+                                 c_void_p,
+                                 BOOL)
 
     # noinspection PyProtectedMember
-    return com_obj(ptr._comobj, byref(interface._iid_), True)
+    return function_pointer(com_ptr._comobj, byref(interface._iid_), True)
 
 
 # noinspection PyUnusedLocal
@@ -347,25 +357,26 @@ def __is_excel_application_empty(excel_app):
     return True
 
 
+# noinspection PyBroadException
 def __move_win32com_gencache_folder():
     """ move win32com gen_py cache files from temp to site-packages folder """
     try:
-        from pathlib import Path
         import shutil
+        import site
         import subprocess
         import win32com
+        from pathlib import Path
 
-        old_gcf = __default_gencache_folder()
-        new_gcf = __site_gencache_folder()
+        old_gcf = os.environ['userprofile'] + '\\AppData\\Local\\Temp\\gen_py\\'
+        new_gcf = site.getsitepackages()[1] + '\\win32com\\gen_py\\'
 
         if not os.path.exists(old_gcf):
             old_gcf = win32com.__gen_path__
-            if old_gcf == new_gcf:
+            if standardize_path(old_gcf) == standardize_path(new_gcf):
                 return
 
-        s = styled(vengeance_message('Attempting to reset win32com gencache folder ...\n'),
-                   'yellow',
-                   'bold')
+        s = vengeance_message('Attempting to reset win32com gencache folder ...\n')
+        s = styled(s, 'yellow', 'bold')
         print(s)
 
         if os.path.exists(old_gcf):
@@ -378,13 +389,3 @@ def __move_win32com_gencache_folder():
 
     except Exception:
         pass
-
-
-def __default_gencache_folder():
-    return os.environ['userprofile'] + '\\AppData\\Local\\Temp\\gen_py\\'
-
-
-def __site_gencache_folder():
-    import site
-    return site.getsitepackages()[1] + '\\win32com\\gen_py\\'
-
