@@ -46,12 +46,10 @@ def parse_timedelta(td):
     if not isinstance(td, timedelta):
         raise TypeError('value is not timedelta instance')
 
-    ParsedTimeDelta = namedtuple('ParsedTimeDelta', 'days hours minutes seconds')
-
-    # s = td.seconds
-    # s += td.days * 24 * 60 * 60
-    # mis = td.microseconds
-
+    ParsedTimeDelta = namedtuple('ParsedTimeDelta', ('days',
+                                                     'hours',
+                                                     'minutes',
+                                                     'seconds'))
     s = td.total_seconds()
     m, s = divmod(s, 60)
     h, m = divmod(m, 60)
@@ -122,7 +120,7 @@ def parse_date_string(s, d_format):
             date_time = __parse_date_dateutil(s)
         else:
             from .text import vengeance_message
-            vengeance_message("('python-dateutil' package not installed)")
+            print(vengeance_message("('python-dateutil' package not installed)"))
 
     return date_time
 
